/* Main header for OperStats

   OperStats - Copyright (c) 2000-2002 Darcy Grexton
   Contact: skold@habber.net, skold @ HabberNet, ICQ #6285178

   This program is free but copyrighted software; see the file COPYING
   for details.
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <setjmp.h>
#include <sys/stat.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <grp.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include "config.h"
#include "sysconf.h"
#include "response.h"

/* True or False? This is used for toggles mainly. */
typedef enum {ERROR = -1,FALSE, TRUE} Boolean_T;

/* Flags */
#define RUN_LIVE	0x00000001	/* Don't fork		*/
#define RUN_QUITTING	0x00000002	/* Shutting down	*/
#define RUN_STARTING	0x00000004	/* Starting		*/
#define RUN_STARTED	0x00000008	/* Started		*/
#define RUN_TERMINATING	0x00000010	/* Terminating		*/
#define RUN_SIGTERM	0x00000020	/* Sigterm		*/
#define RUN_SAVEDBS	0x00000040	/* Save DBs		*/
#define RUN_SHUTDOWN	0x00000080	/* Shut down		*/

/* User Flags */
#define UF_SRA		0x00000001	/* Is an SRA		*/

/* User Modes - Only modes we care about */
#define UMODE_o		0x00000001	/* Opered		*/

/* Channel Modes - Only modes we care about */
#define CMODE_s		0x00000001	/* Secret Channel	*/

/* Channel hash */
#define CHASH(chan) ((chan)[1] ? ((chan)[1]&31)<<5 | ((chan)[2]&31) : 0)

/* Nick hash */
#define NHASH(nick) (((nick)[0]&31)<<5 | ((nick)[1]&31))

/* Server hash */
#define SHASH(server) (((server)[0]&31)<<5 | ((server)[1]&31))

/* AKill hash */
#define AHASH(akill) (((akill)[0]&31)<<5 | ((akill)[1]&31))

/* Clone hash */
#define CLHASH(host) (((host)[0]&31)<<5 | ((host)[1]&31))

/* Structure for our stuff */
struct me
{
    char uplink[HOSTLEN];	/* Address we connect to	*/
    int port;			/* Port we do it on		*/
    char pass[PASSWDLEN];	/* Password we use for the link	*/
    char ruplink[HOSTLEN];	/* Real name of our uplink	*/
    char bindhost[HOSTLEN];	/* Address to bind to		*/
    int bindport;		/* Port to bind to		*/
    int token;			/* True if our uplink has tokens*/
    char name[HOSTLEN];		/* Our server name		*/
    char desc[REALLEN];		/* Our description		*/
    time_t since;		/* When we started		*/
    time_t reset;		/* When we last reset		*/
} me;

/* OperServ's information */
struct os
{
    char nick[NICKLEN];		/* Nick we want to have		*/
    char user[USERLEN];		/* OperServ's username		*/
    char host[HOSTLEN];		/* OperServ's hostname		*/
    char mode[12];		/* OperServ's modes		*/
    char real[REALLEN];		/* OperServ's realname		*/
    int isonline;		/* Is OperServ online?		*/
    int wantnick;		/* Are we waiting for our nick?	*/
    int collided;		/* How many collides?		*/
    int version;		/* Version of OperServ's DB	*/
    time_t lastcoll;		/* Time of last collide		*/
} os;

/* StatServ's information */
struct ss
{
    char nick[NICKLEN];		/* Nick we want to have		*/
    char user[USERLEN];		/* StatServ's username		*/
    char host[HOSTLEN];		/* StatServ's hostname		*/
    char mode[12];		/* StatServ's modes		*/
    char real[REALLEN];		/* StatServ's realname		*/
    int isonline;		/* Is StatServ online?		*/
    int wantnick;		/* Are we waiting for our nick?	*/
    int collided;		/* How many collides?		*/
    int version;		/* Version of StatServ's DB	*/
    time_t lastcoll;		/* Time of last collide		*/
} ss;

/* GlobalNoticer's information */
struct gn
{
    char nick[NICKLEN];		/* Nick we want to have		*/
    char user[USERLEN];		/* GlobalNoticer's username	*/
    char host[HOSTLEN];		/* GlobalNoticer's hostname	*/
    char mode[12];		/* GlobalNoticer's modes	*/
    char real[REALLEN];		/* GlobalNoticer's realname	*/
    int isonline;		/* Is the GlobalNoticer online? */
    int wantnick;		/* Are we waiting for our nick?	*/
    int collided;		/* How many collides?		*/
    time_t lastcoll;		/* Time of last collide		*/
} gn;

/* SpamServ's information */
struct sp
{
    char nick[NICKLEN];		/* SpamServ's nickname		*/
    char user[USERLEN];		/* SpamServ's username		*/
    char host[HOSTLEN];		/* SpamServ's hostname		*/
    char real[REALLEN];		/* SpamServ's realname		*/
    int isonline;		/* Is SpamServ online?		*/
    time_t lastnick;		/* Time of last nick change	*/
} sp;

typedef struct user_ User;
typedef struct channel_ Channel;
typedef struct sra_ SRA;
typedef struct tld_ TLD;
typedef struct zone_ Zone;
typedef struct msg_ MSG;
typedef struct servstat_ ServStat;
typedef struct akill_ AutoKill;
typedef struct clone_ Clone;
typedef struct trigger_ Trigger;
typedef struct exception_ Exception;
typedef struct uline_ ULine;

/* User structure: One per user */
struct user_ {
    User *next, *prev;
    char nick[NICKLEN];		/* NickName			*/
    char *user;			/* UserName			*/
    char *host;			/* HostName			*/
    char *real;			/* RealName			*/
    char *server;		/* Server			*/
    int32 mode;			/* UserModes			*/
    int32 flags;		/* User Flags			*/
    struct u_chanlist {		/* Channels user is on		*/
	struct u_chanlist *next, *prev;
	Channel *chan;		/* Channel Name			*/
    } *chans;
};

/* Channel structure: One per channel */
struct channel_ {
    Channel *next, *prev;
    char name[CHANNELLEN];	/* Channel Name			*/
    char *topic;		/* Channel's Topic		*/
    int32 mode;			/* Channel Modes		*/
    struct c_userlist {		/* Users on channel		*/
	struct c_userlist *next, *prev;
	User *user;		/* User's NickName		*/
    } *users;
};

/* SRAs */
struct sra_ {
    SRA *next, *prev;
    char *nick;			/* SRA's nickname		*/
    char *pass;			/* SRA's password		*/
};

/* Clone Triggers */
struct trigger_ {
    Trigger *next, *prev;
    char host[HOSTLEN];
    int limit;
};

/* Clone Exceptions */
struct exception_ {
    Exception *next, *prev;
    char host[HOSTLEN];
};

/* Top-Level-Domain list */
struct tld_ {
    TLD *next, *prev;
    char *name;			/* The actual TLD		*/
    char *desc;			/* Text description of TLD	*/
    int cnt;			/* Number of clients from TLD	*/
    int hits;			/* Daily hits to this TLD	*/
};

/* TimeZone list */
struct zone_ {
    Zone *next, *prev;
    char *name;			/* TimeZone Name (ie, PST)	*/
    char *noffset;		/* Named offset (ie, -0800)	*/
    char *desc;			/* Full Description		*/
    int offset;			/* GMT Offset in seconds	*/
};

/* LogonMSGs */
struct msg_ {
    MSG *next, *prev;
    char *text;			/* Body of the LogonMSG		*/
    char *setter;		/* Who set the LogonMSG		*/
    time_t set;			/* When it was set		*/
};

/* Server information */
typedef struct {
    char *server;		/* Server's name		*/
    char *uplink;		/* Server's uplink		*/
    time_t connect;		/* When it connected		*/
    struct timeval lastping;	/* When it was last pinged	*/
    struct timeval lastpong;	/* When it replied to a ping	*/
    int hops;			/* How many hops away from us?	*/
} Servers;

/* Server statistics info */
struct servstat_ {
    ServStat *next, *prev;
    char *name;			/* Server's name		*/
    char *version;		/* Server's version reply	*/
    char *flags;		/* Flags from version reply	*/
    char *lastquit;		/* Server's last quit message	*/
    time_t statssince;		/* How far back the stats go	*/
    time_t lastsplit;		/* When it last split		*/
    time_t lastreport;		/* When we said it was lagged	*/
    int is_uline;		/* Is this server U:Lined?	*/
    int is_split;		/* Is it split right now?	*/
    int splittime;		/* When did it split?		*/
    int lastdowntime;		/* How long it was split last	*/
    int penalize;		/* Was it the source of a split?*/
    int splits;			/* How many times has it split?	*/
    struct timeval lag;		/* Current lag			*/
    struct timeval peaklag;	/* Peak lag			*/
};

/* AKill structure */
struct akill_ {
    AutoKill *next, *prev;
    char *mask;			/* Mask that's AKilled		*/
    char *reason;		/* Reason for the AKill		*/
    char *setter;		/* Who set the AKill		*/
    int realname;		/* Is it a realname AKill?	*/
    time_t set;			/* When the AKill was set	*/
    time_t expires;		/* When the AKill expires	*/
};

/* Clone structure */
struct clone_ {
    Clone *next, *prev;
    char host[HOSTLEN];		/* Host for this record		*/
    int cnt;			/* Number of connections	*/
    int kills;			/* Number of kills		*/
    time_t lastkill;		/* When we last killed this host*/
};

/* U:Lined servers */
struct uline_ {
    ULine *next, *prev;
    char *name;
};

/* Commands we understand: See process.c */
typedef struct {
    const char *name;
    void (*func) (char *source, int ac, char **av);
} Message;

extern Message messages[];
extern Message *find_message (const char *name);

/* Command hash */
typedef struct {
    char *accept;
    short access;
    void (*process) (const char *source);
} Hash;

/* Subcommand hash */
typedef struct {
    char *accept;
    short access;
    void (*process) (const char *source);
} Hash_SUB;

/* Help hash */
typedef struct {
    char *accept;
    short access;
    void (*process) (const char *source);
} Hash_HELP;

extern Hash *get_hash (const char *whoami, const char *source, const char *cmd,
		       Hash *hash_table); 
extern Hash_SUB *get_sub_hash (const char *whoami, const char *source, 
			       const char *cmd, Hash_SUB *hash_table);
extern Hash_HELP *get_help_hash (const char *whoami, const char *source,
				 const char *cmd, Hash_HELP *hash_table);

/* Hash table access levels */
#define H_NONE  0
#define H_IRCOP 1
#define H_SRA   2

/* This needs to be down here so the stuff in operstats.h it uses works */
#include "extern.h"
